@@ -1,5 +1,27 @@
+__all__ = [
+    "ParentException",
+    "CaptchaException",
+    "SolverError",
+    "LoginError",
+    "UserError",
+    "PlaylistError",
+    "SaverError",
+    "SongError",
+    "ArtistError",
+    "BaseClientError",
+    "RequestError",
+    "GeneratorError",
+    "PasswordError",
+    "FamilyError",
+    "WebSocketError",
+    "PlayerError",
+    "AlbumError",
+    "PodcastError",
+]
+
+
 class ParentException(Exception):
-    def __init__(self, message: str, error: str = str) -> None:
+    def __init__(self, message: str, error: str | None = None) -> None:
         super().__init__(message)
         self.error = error
 
@@ -64,5 +86,26 @@ class PasswordError(ParentException):
     pass
 
 
+# family.py exceptions
 class FamilyError(ParentException):
     pass
+
+
+# websocket.py exceptions
+class WebSocketError(ParentException):
+    pass
+
+
+# player.py exceptions
+class PlayerError(ParentException):
+    pass
+
+
+# album.py exceptions
+class AlbumError(ParentException):
+    pass
+
+
+# podcast.py exceptions
+class PodcastError(ParentException):
+    pass
